#!/usr/bin/perl
#
# (C) 2001-2002
#	Domsodi Gergely	<doome@uhulinux.hu>
#	Pozsar Balazs	<pozsy@uhulinux.hu>
#
# This file is originally part of the UHU Linux distribution.
# Distributable under GPL v2.
#

# ----- Pragmatic modules -----------------------------------------------------
use strict;
use warnings;

# -----------------------------------------------------------------------------
package UHU::common;

use POSIX ":sys_wait_h";

# ----- Exporter --------------------------------------------------------------
require Exporter;
our @ISA    = qw(Exporter);
our @EXPORT = qw(
debug
chomp_
cat cat1 ccat put
default_env
nicesize nicetime
get_hashkey
fsize
splitup
sortn

system__ system_
fork__ fork_

basename dirname mkdir_ ln pwd

takeElement isElement

target

wait_ r
);

# -----------------------------------------------------------------------------
sub debug($) {
	my ($msg) = @_;

	print STDERR $msg . "\n" if $ENV{UHU_DEBUG};
}

sub depricated($) {
	my ($msg) = @_;

	print STDERR $msg . "\n";
	die;
}

sub fat_warning($) {
	my ($msg) = @_;

	print STDERR "!!!" . $msg . "!!!\n";
}

# -----------------------------------------------------------------------------
sub chomp_ {
	$_ and chomp $_ for @_;
	wantarray ? @_ : join '', @_;
}

# -----------------------------------------------------------------------------
sub cat {
	local @ARGV = @_;

	wantarray ? <> : join '', <>;
}

sub cat1 {
	local @ARGV = @_;

	my $str = <>;
	chomp_ $str;
}

sub _put($$) {
	my ($data, $filename) = @_;

	open(F, ">" . $filename) or return -1;
	print F $data;
	close F;
}

sub put($$) {
	_put($_[0], $_[1]) or die "Nem lehet megnyitni irsra: [$_]";
}

# -----------------------------------------------------------------------------
sub default_env($$) {
	my ($key, $value) = @_;

	$ENV{$key} = $value unless $ENV{$key};
}

# -----------------------------------------------------------------------------
sub pad($) {
	my ($val) = @_;
	return ($val < 10 ? "0" : "") . $val;
}

sub nicesize($) {
	my ($size) = @_;

	return '0 B' unless $size;
	return (int($size)) . " B" if $size < 1024;
	return (int($size * 10 / 1024**1) / 10) . " KB" if $size < 1024**2;
	return (int($size * 10 / 1024**2) / 10) . " MB" if $size < 1024**3;
	return (int($size * 10 / 1024**3) / 10) . " GB";
}

sub nicetime($) {
	my ($sex) = @_;

	($sex > 24 * 3600) and return 'ezt nem fogod kivrni :-)';
	my $hours = int($sex / 3600);
	$sex -= $hours * 3600;
	my $minutes = int($sex / 60);
	$sex -= $minutes * 60;

	return (pad $hours) . ':' . (pad $minutes) . ':' . (pad $sex);
}

# -----------------------------------------------------------------------------
sub get_hashkey($$) {
	my ($hash, $value) = @_;

	({ reverse %$hash }->{$value});
}

sub takeElement($$) {
	my ($element, $array) = @_;

	[map {($element eq $_) ? () : $_} @{ $array or [] }];
}

sub isElement($$) {
	my ($element, $array) = @_;
	($element eq $_) and return 1 foreach @{ $array or [] };
}

# -----------------------------------------------------------------------------
sub splitup($) {
	my ($str) = @_;

	my @ret;
	while ($str ne '') {
		push @ret, $1 if ($str =~ s/^(\d+|\D+)//);
	}

	@ret;
}

sub sortn {
	sort {
		my $x;
		my $y;
		my @a = splitup $a;
		my @b = splitup $b;
		while (@a and @b) {
			($x, $y) = (shift @a, shift @b);
			if ($x =~ /\d+/) {
				if ($y =~ /\d+/) {
					return $x <=> $y if $x <=> $y;
					return $x cmp $y if $x cmp $y;
				} else {
					return -1;
				}
			} else {
				if ($y =~ /\d+/) {
					return 1;
				} else {
					return $x cmp $y if $x cmp $y;
				}
			}
		}
		@a <=> @b;
	} @_;
}

# -----------------------------------------------------------------------------
sub _chroot() {
	($ENV{TARGET} eq '/') ? () : ('chroot', $ENV{TARGET});
}

sub fork__(@) {
	my $pid;

	debug "fork__[@_]";
	unless ($pid = fork) {
		setpgrp 0, 0;
		exec @_ or POSIX::_exit(-1);
	}

	$pid;
}

sub fork_(@) {
	fork__(_chroot, @_);
}

sub system__(@) {
	debug "system__[@_]";
	system @_;
}

sub system_(@) {
	system__(_chroot, @_);
}

# -----------------------------------------------------------------------------
sub basename($) {
	($_) = @_;

	$_ or return;

	s|/$||;
	s|^.*/||;
	$_ or '/';
}

sub dirname($) {
	($_) = @_;

	$_ or return '.';

	s|/[^/]*$|| or return '.';
	$_ or '.';
}

sub mkdir_($) {
	my ($dir) = @_;

	system 'mkdir', '-p', $dir;
}

sub ln($$) {
	my ($orig, $new) = @_;

	mkdir_ dirname $new;
	system 'ln', '-s', $orig, $new;
}

sub wait_() {
	waitpid -1, &WNOHANG;
}

sub r() {
	$?>>8;
}

# -----------------------------------------------------------------------------
sub target() {
	$ENV{TARGET};
}

# ----- Initialization --------------------------------------------------------
default_env 'TARGET',   '/';
default_env 'UHU_ROOT', '/';

# ----- Beatiful perl :) ------------------------------------------------------
"Ide valami viccet kene irni, nem?";
